<?php
/**
 * eMySQLAdmin
 *
 * en : Class witch interact with MySQL server
 * @author AIT YAHIA Idir
 * @version 0.1
 * @license    Open source Licence
 * 
 */
class dbOperation {
	
	protected $db;
	
	public function __construct($dbAdapter) {
		$this->db = $dbAdapter;
	}
	
	/**
	 * en : get database list form server
	 *
	 * @param  void
	 * @return array Ext.TreeNode a database list
	 */
	function getDatabase() {
		$sql = 'SHOW DATABASES';
		$rec = $this->db->fetchAll ( $sql );
		$data = Array ();
		foreach ( $rec as $value ) {
			array_push ( $data, array ('id' => $value ['Database'], 'text' => $value ['Database'], 'iconCls' => 'db', 'objectType' => 'database' ) );
		}
		
		return $data;
	}
	
	/**
	 * en : get table fields
	 * @param  string $database a database name
	 * @param  string $table a table name
	 * @return array Ext.TreeNode  
	 */
	function getFields($database, $table) {
		$select = $this->db->select ()->from ( 'information_schema.COLUMNS', array ('COLUMN_NAME', 'DATA_TYPE' ) )->where ( 'TABLE_SCHEMA = ?', $database )->where ( 'TABLE_NAME = ?', $table );
		
		$rec = $this->db->fetchAll ( $select );
		
		$data = array ();
		foreach ( $rec as $value ) {
			
			array_push ( $data, array ('id' => $database . '-' . $table . '-' . $value ['COLUMN_NAME'] . '-field', 'text' => $value ['COLUMN_NAME'], 'leaf' => true, 'iconCls' => $value ['DATA_TYPE'], 'objectType' => 'field', 'table' => $table, 'database' => $database ) );
		
		}
		
		$fields = array ('id' => $database . '-' . $table . '-fields', 'text' => 'Fields', 'children' => $data );
		
		return $fields;
	
	}
	
	/**
	 * en : get table indexes
	 *
	 * @param  string $database a database name
	 * @param  string $table a table name
	 * @return array Ext.TreeNode  
	 */
	function getIndexe($database, $table) {
		$select = $this->db->select ()->from ( 'information_schema.STATISTICS', array ('INDEX_NAME', 'SEQ_IN_INDEX', 'COLUMN_NAME' ) )->where ( 'TABLE_SCHEMA = ?', $database )->where ( 'TABLE_NAME = ?', $table )->group ( 'INDEX_NAME' );
		
		$rec = $this->db->fetchAll ( $select );
		
		$data = array ();
		foreach ( $rec as $value ) {
			$cls = ($value ['INDEX_NAME'] == 'PRIMARY') ? 'PRIMARY' : 'table_key';
			
			array_push ( $data, array ('id' => $database . '-' . $table . '-' . $value ['INDEX_NAME'] . '-indexe', 'text' => $value ['INDEX_NAME'], 'leaf' => true, 'iconCls' => $cls, 'objectType' => 'index', 'table' => $table, 'database' => $database ) );
		
		}
		
		$fields = array ('text' => 'Indexes', 'children' => $data );
		
		return $fields;
	
	}
	
	/**
	 * en : get tables list of specified database.
	 *
	 * @param  string $database a database name.
	 * @return array Ext.TreeNode a tables list.  
	 */
	
	function getTables($database) {
		
		$select = $this->db->select ()->from ( 'information_schema.TABLES', array ('TABLE_NAME', 'TABLE_ROWS' ) )->where ( 'TABLE_TYPE = ?', 'BASE TABLE' )->where ( 'TABLE_SCHEMA = ?', $database );
		
		$rec = $this->db->fetchAll ( $select );
		
		$data = array ();
		foreach ( $rec as $value ) {
			$fields = $this->getFields ( $database, $value ['TABLE_NAME'] );
			$indexes = $this->getIndexe ( $database, $value ['TABLE_NAME'] );
			array_push ( $data, array ('id' => $database . '-' . $value ['TABLE_NAME'] . '-table', 'text' => $value ['TABLE_NAME'] . ' (' . $value ['TABLE_ROWS'] . ')', 'iconCls' => 'tables', 'objectType' => 'table', 'database' => $database, 'children' => array ($fields, $indexes ) ) );
		
		}
		
		$table = array ('id' => $database . '-tables', 'text' => 'Tables', 'children' => $data );
		return $table;
	}
	
	/**
	 * en : get stored procedure of specified database.
	 * @param  string $database a database name.
	 * @return array Ext.TreeNode a stored proc list.  
	 */
	function getProcedures($database) {
		
		$select = $this->db->select ()->from ( 'information_schema.ROUTINES', 'SPECIFIC_NAME' )->where ( 'ROUTINE_TYPE = ?', 'PROCEDURE' )->where ( 'ROUTINE_SCHEMA = ?', $database );
		
		$rec = $this->db->fetchAll ( $select );
		
		$data = array ();
		foreach ( $rec as $value ) {
			array_push ( $data, array ('id' => $database . '-' . $value ['SPECIFIC_NAME'] . '-Procedure', 'text' => $value ['SPECIFIC_NAME'], 'iconCls' => 'procedure', 'database' => $database, 'objectType' => 'procedure' ) );
		
		}
		
		$Procedures = array ('id' => $database . '-Procedures', 'text' => 'Procedures', 'children' => $data );
		
		return $Procedures;
	
	}
	
	/**
	 * en : get stored functions of specified database.
	 *
	 * @param  string $database a database name.
	 * @return array Ext.TreeNode a stored function list.  
	 */
	
	function getfunctions($database) {
		$select = $this->db->select ()->from ( 'information_schema.ROUTINES', 'SPECIFIC_NAME' )->where ( 'ROUTINE_TYPE = ?', 'FUNCTION' )->where ( 'ROUTINE_SCHEMA = ?', $database );
		
		$rec = $this->db->fetchAll ( $select );
		
		$data = array ();
		foreach ( $rec as $value ) {
			array_push ( $data, array ('id' => $database . '-' . $value ['SPECIFIC_NAME'] . '-function', 'text' => $value ['SPECIFIC_NAME'], 'iconCls' => 'procedure', 'database' => $database, 'objectType' => 'function' ) );
		
		}
		
		$functions = array ('id' => $database . '-functions', 'text' => 'Fonctions', 'children' => $data );
		
		return $functions;
	
	}
	
	/**
	 * en : get triggers of specified database.
	 *
	 * @param  string $database a database name.
	 * @return array Ext.TreeNode a triggers list.  
	 */
	
	function getTriggers($database) {
		$select = $this->db->select ()->from ( 'information_schema.TRIGGERS', 'TRIGGER_NAME' )->where ( 'TRIGGER_SCHEMA = ?', $database );
		
		$rec = $this->db->fetchAll ( $select );
		
		$data = array ();
		foreach ( $rec as $value ) {
			array_push ( $data, array ('id' => $database . '-' . $value ['TRIGGER_NAME'] . '-Trigger', 'text' => $value ['TRIGGER_NAME'], 'iconCls' => 'trigger', 'database' => $database, 'objectType' => 'triggers' ) );
		
		}
		
		$triggers = array ('id' => $database . '-Triggers', 'text' => 'Triggers', 'children' => $data );
		
		return $triggers;
	
	}
	
	/**
	 * en : get views list of specified database.
	 *
	 * @param  string $database a database name.
	 * @return array Ext.TreeNode a views list.  
	 */
	
	function getViews($database) {
		$select = $this->db->select ()->from ( 'information_schema.VIEWS', 'TABLE_NAME' )->where ( 'TABLE_SCHEMA = ?', $database );
		
		$rec = $this->db->fetchAll ( $select );
		
		$data = array ();
		foreach ( $rec as $value ) {
			array_push ( $data, array ('id' => $database . '-' . $value ['TABLE_NAME'] . '-Views', 'text' => $value ['TABLE_NAME'], 'iconCls' => 'views', 'database' => $database, 'objectType' => 'views' ) );
		
		}
		
		$views = array ('id' => $database . '-Views', 'text' => 'Views', 'children' => $data );
		
		return $views;
	
	}
	
	function getServerCharset() {
		
		$encodage = mysql_client_encoding ();
		$select = $this->db->select ()->from ( 'information_schema.CHARACTER_SETS', 'DESCRIPTION' )->where ( 'CHARACTER_SET_NAME = ?', $encodage );
		$rec = $this->db->fetchOne ( $select );
		
		return $rec . ' (' . $encodage . ')';
	
	}
	
	function getCharset() {
		
		$select = $this->db->select ()->from ( 'information_schema.CHARACTER_SETS', array ('CHARACTER_SET_NAME', 'DESCRIPTION', 'DEFAULT_COLLATE_NAME' ) );
		$rec = $this->db->fetchAll ( $select );
		
		return $rec;
	
	}
	
	function getCollate($charset) {
		
		$select = $this->db->select ()->from ( 'information_schema.COLLATIONS', array ('CHARACTER_SET_NAME', 'COLLATION_NAME' ) )->where ( 'CHARACTER_SET_NAME = ?', $charset );
		$rec = $this->db->fetchAll ( $select );
		
		return $rec;
	
	}

}

